//
// MyView.m
// Core Animation Screen Cast
//
// Created by Bill Dudney on 05/01/08.
//
// Copyright 2008 Gala Factory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#import "MyView.h"
#import <QuartzCore/QuartzCore.h>
#import "CGColorHolder.h"
#import "MyLayoutManager.h"

#define SCALE 0.75f

NSString *kImageLayerKey = @"__imageLayer";
NSString *kMaxAvalibleImageWidth = @"maxAvalibleImageWidth";
NSString *kMaxAvalibleImageHeight = @"maxAvalibleImageHeight";
@interface MyView (Private)

- (void)addImageSublayers;

- (CGImageRef)loadImage:(NSString*)name;
- (CGRect)bestFitForImage:(CGImageRef) image;

- (NSNumber *)maxAvalibleImageWidth;
- (NSNumber *)maxAvalibleImageHeight;

@end

@implementation MyView

- (void)awakeFromNib {
	self.layer = [CALayer layer];
	self.wantsLayer = YES;
	self.layer.layoutManager = [CAConstraintLayoutManager layoutManager];
	self.layer.backgroundColor = [CGColorHolder black];

	contentLayer = [CALayer layer];
	//contentLayer.borderWidth = 2.0f;
	//contentLayer.borderColor = [CGColorHolder red];
	contentLayer.backgroundColor = [CGColorHolder black];
	contentLayer.layoutManager = [MyLayoutManager layoutManager];
	contentLayer.constraints = [NSArray arrayWithObjects:[CAConstraint constraintWithAttribute:kCAConstraintMidX relativeTo:@"superlayer" attribute:kCAConstraintMidX], [CAConstraint constraintWithAttribute:kCAConstraintMidY relativeTo:@"superlayer" attribute:kCAConstraintMidY], nil];
	[contentLayer setValue:[self maxAvalibleImageWidth] 
					forKey:kMaxAvalibleImageWidth];
	[contentLayer setValue:[self maxAvalibleImageHeight] 
					forKey:kMaxAvalibleImageHeight];

	[self.layer addSublayer:contentLayer];
	
	[self addImageSublayers];
}

- (BOOL)acceptsFirstResponder {
	return YES;
}

- (void)moveUp:(id)sender {
	NSArray *sublayers = [contentLayer sublayers];
	NSUInteger sublayerCount = [sublayers count];
	CALayer *sublayer = [sublayers objectAtIndex:sublayerCount - 1];
	[sublayer removeFromSuperlayer];
	[contentLayer insertSublayer:sublayer atIndex:0];
}

- (void)moveDown:(id)sender {
	NSArray *sublayers = [contentLayer sublayers];
	NSUInteger sublayerCount = [sublayers count];
	CALayer *sublayer = [sublayers objectAtIndex:0];
	[sublayer removeFromSuperlayer];
	[contentLayer insertSublayer:sublayer atIndex:sublayerCount - 1];
}

@end

@implementation MyView (Private)

- (CALayer *)layerForImageNamed:(NSString *)name {
	CALayer *containerLayer = [CALayer layer];
	containerLayer.name = [NSString stringWithFormat:@"Container_%@", name];
	containerLayer.layoutManager = [CAConstraintLayoutManager layoutManager];

	CALayer *imageLayer = [CALayer layer];
	imageLayer.contents = (id)[self loadImage:name];
	imageLayer.bounds = 
	CGRectMake(0.0f, 0.0f, 
			   CGImageGetWidth((CGImageRef)imageLayer.contents), 
			   CGImageGetHeight((CGImageRef)imageLayer.contents));
	imageLayer.name = name;
	imageLayer.contentsGravity = kCAGravityResizeAspectFill;
	imageLayer.masksToBounds = YES;
	imageLayer.constraints = [NSArray arrayWithObjects:[CAConstraint constraintWithAttribute:kCAConstraintMidX relativeTo:@"superlayer" attribute:kCAConstraintMidX], [CAConstraint constraintWithAttribute:kCAConstraintMidY relativeTo:@"superlayer" attribute:kCAConstraintMidY], nil];
	
	[containerLayer setValue:imageLayer forKey:kImageLayerKey];
	//containerLayer.sublayerTransform = CATransform3DMakeRotation(45.0f * M_PI / 180.0f, 0.0f, 1.0f, 0.0f);
	[containerLayer addSublayer:imageLayer];
	
	return containerLayer;
}

- (void)addImageSublayers {
	NSArray *imageNames = [NSArray arrayWithObjects:@"GettyMuseum", 
						   @"CreepyHollywoodHotel", @"SunThroughSnowClouds", 
						   @"TenMileRange", @"GoldenGateBridge", nil];
	for(NSString *imageName in imageNames) {
		[contentLayer addSublayer:[self layerForImageNamed:imageName]];
	}
}	

- (CGRect) bestFitForImage:(CGImageRef)image {
	CGSize size = CGSizeMake([[self maxAvalibleImageWidth] floatValue], 
							[[self maxAvalibleImageHeight] floatValue]);
	CGFloat imageWidth = CGImageGetWidth(image);
	CGFloat imageHeight = CGImageGetHeight(image);
	CGFloat scale = 0.0f;
	if(size.width / imageWidth < size.height / imageHeight) {
		scale = size.width / imageWidth;
	} else {
		scale = size.height / imageHeight;
	}
	CGRect rect = {CGPointZero, {scale * imageWidth, scale * imageHeight}};
	// if we have a portrait image we want to center it on the width
	if(rect.size.width < size.width) {
		rect.origin.x = (size.width - rect.size.width)/2.0f;
	}
	return rect;
}

- (CGImageRef)loadImage:(NSString*)name {
	CGImageRef image = NULL;
	NSBundle *bun = [NSBundle mainBundle];
	NSString *path = [bun pathForResource:name ofType:@"jpg"];
	NSURL * url = [NSURL fileURLWithPath:path];
	CGImageSourceRef imageSource = CGImageSourceCreateWithURL((CFURLRef)url, NULL);
	if(imageSource != NULL) {
		image = CGImageSourceCreateImageAtIndex(imageSource, 0, NULL);
		CFRelease(imageSource);
	} else {
		NSLog(@"Could not load image");
		return NULL;
	}
	
	if(NULL != image) {
		CGRect rect = [self bestFitForImage:image];
		CGColorSpaceRef colorSpace = 
			CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
		CGContextRef context = CGBitmapContextCreate(NULL, rect.size.width, 
													 rect.size.height, 8, 0, 
													 colorSpace, 
													 kCGImageAlphaPremultipliedFirst); 
		CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
		rect.origin = CGPointZero;
		CGContextDrawImage(context, rect, image);
		CGContextFlush(context);
		CGImageRef scaledImage = CGBitmapContextCreateImage(context);
		CGContextRelease (context);
		CGImageRelease(image);
		image = NULL;
		image = scaledImage;
	}
	
	return image;
}

- (NSNumber *)maxAvalibleImageWidth {
	return [NSNumber numberWithFloat:CGRectGetWidth(self.layer.bounds) * SCALE];
}

- (NSNumber *)maxAvalibleImageHeight {
	return [NSNumber numberWithFloat:CGRectGetHeight(self.layer.bounds) * SCALE];
}

@end
